<?PHP
/**
 * Represents a type of page for which custom reports can be generated in musdb.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);

/**
 * Represents a type of page for which custom reports can be generated in musdb.
 */
enum MDCustomReportTarget implements MDValueEnumInterface, JsonSerializable {

    case object_single;
    case object_list;
    case loan;
    case exhibition;

    /**
     * Returns a value of this type based on a string.
     *
     * @param string $input Input to get a value from.
     *
     * @return MDCustomReportTarget
     */
    public static function fromString(string $input):MDCustomReportTarget {

        return match($input) {
            'object_single' => self::object_single,
            'object_list' => self::object_list,
            'loan' => self::loan,
            'exhibition' => self::exhibition,
            default => throw new MDpageParameterNotFromListException("Unknown custom report target: " . $input),
        };

    }

    /**
     * Lists all available names.
     *
     * @return array<string>
     */
    public static function caseNames():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->name;
        }

        return $output;

    }

    /**
     * Gets an unsorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getUnsortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlUnsortedList($tlLoader, self::caseNames(), "custom_report_targets_set", "custom_report_targets_set");

    }

    /**
     * Gets a sorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getSortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlSortedList($tlLoader, self::caseNames(), "custom_report_targets_set", "custom_report_targets_set");

    }

    /**
     * Returns the name of the current value in translation.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return string
     */
    public function getTledName(MDTlLoader $tlLoader):string {

        return $tlLoader->tl("custom_report_targets_set", "custom_report_targets_set", $this->name);

    }

    /**
     * Provides the option to serialize as a string during json_encode().
     *
     * @return string
     */
    public function jsonSerialize():string {

        return $this->name;

    }
}
