<?PHP
/**
 * Represents a type of collection focus a museum (or similar institution) may have.
 * This can be useful for statistics and be used in line with institutions' self-
 * categorization.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);

/**
 * Represents a type of collection focus a museum (or similar institution) may have.
 * This can be useful for statistics and be used in line with institutions' self-
 * categorization.
 */
enum MDInstitutionCollectionArea implements MDValueEnumInterface, JsonSerializable {

    case everyday_life;
    case archaeology;
    case architecture;
    case history_of_mining;
    case botany;
    case history_of_film;
    case political_history;
    case arts;
    case applied_arts;
    case agriculture;
    case literature;
    case history_of_medicine;
    case military_history;
    case mineralogy;
    case music;
    case numismatics;
    case paleontology;
    case personalia;
    case regional_history;
    case religion;
    case technology;
    case ethnography;
    case zoology;

    /**
     * Returns a value of this type based on a string.
     *
     * @param string $input Input to get a value from.
     *
     * @return MDInstitutionCollectionArea
     */
    public static function fromString(string $input):MDInstitutionCollectionArea {

        return match($input) {
            "everyday_life" => self::everyday_life,
            "archaeology" => self::archaeology,
            "architecture" => self::architecture,
            "history_of_mining" => self::history_of_mining,
            "botany" => self::botany,
            "history_of_film" => self::history_of_film,
            "political_history" => self::political_history,
            "arts" => self::arts,
            "applied_arts" => self::applied_arts,
            "agriculture" => self::agriculture,
            "literature" => self::literature,
            "history_of_medicine" => self::history_of_medicine,
            "military_history" => self::military_history,
            "mineralogy" => self::mineralogy,
            "music" => self::music,
            "numismatics" => self::numismatics,
            "paleontology" => self::paleontology,
            "personalia" => self::personalia,
            "regional_history" => self::regional_history,
            "religion" => self::religion,
            "technology" => self::technology,
            "ethnography" => self::ethnography,
            "zoology" => self::zoology,
            default => throw new MDpageParameterNotFromListException("Unknown collection focus area"),
        };

    }

    /**
     * Returns instances of MDInstitutionCollectionArea based on list of strings.
     *
     * @param list<string> $input Input to get a value from.
     *
     * @return list<MDInstitutionCollectionArea>
     */
    public static function fromStrings(array $input):array {

        $output = [];
        foreach ($input as $value) {
            $output[] = self::fromString($value);
        }
        return $output;

    }

    /**
     * Returns a value of this type based on an integer.
     *
     * @param integer $input Input to get a value from.
     *
     * @return MDInstitutionCollectionArea
     */
    public static function fromInt(int $input):MDInstitutionCollectionArea {

        return match($input) {
            0 => self::everyday_life,
            1 => self::archaeology,
            2 => self::architecture,
            3 => self::history_of_mining,
            4 => self::botany,
            5 => self::history_of_film,
            6 => self::political_history,
            7 => self::arts,
            8 => self::applied_arts,
            9 => self::agriculture,
            10 => self::literature,
            11 => self::history_of_medicine,
            12 => self::military_history,
            13 => self::mineralogy,
            14 => self::music,
            15 => self::numismatics,
            16 => self::paleontology,
            17 => self::personalia,
            18 => self::regional_history,
            19 => self::religion,
            20 => self::technology,
            21 => self::ethnography,
            22 => self::zoology,
            default => throw new MDpageParameterNotFromListException("Unknown collection focus area"),
        };

    }

    /**
     * Lists all available names.
     *
     * @return array<string>
     */
    public static function caseNames():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->name;
        }

        return $output;

    }

    /**
     * Gets an unsorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getUnsortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlUnsortedList($tlLoader, self::caseNames(), "institution_collection_area_set", "institution_collection_area_set");

    }

    /**
     * Gets a sorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getSortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlSortedList($tlLoader, self::caseNames(), "institution_collection_area_set", "institution_collection_area_set");

    }

    /**
     * Lists all available names.
     *
     * @return array<integer>
     */
    public static function caseInts():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->toInt();
        }

        return $output;

    }

    /**
     * Returns integer representation of object record status.
     *
     * @return integer
     */
    public function toInt():int {

        return match($this) {
            self::everyday_life => 0,
            self::archaeology => 1,
            self::architecture => 2,
            self::history_of_mining => 3,
            self::botany => 4,
            self::history_of_film => 5,
            self::political_history => 6,
            self::arts => 7,
            self::applied_arts => 8,
            self::agriculture => 9,
            self::literature => 10,
            self::history_of_medicine => 11,
            self::military_history => 12,
            self::mineralogy => 13,
            self::music => 14,
            self::numismatics => 15,
            self::paleontology => 16,
            self::personalia => 17,
            self::regional_history => 18,
            self::religion => 19,
            self::technology => 20,
            self::ethnography => 21,
            self::zoology => 22,
            # default => throw new MDpageParameterNotFromListException("Unknown object record status"),
        };

    }

    /**
     * Returns the name of the current value in translation.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return string
     */
    public function getTledName(MDTlLoader $tlLoader):string {

        return $tlLoader->tl("institution_collection_area_set", "institution_collection_area_set", $this->name);

    }

    /**
     * Provides the option to serialize as a string during json_encode().
     *
     * @return string
     */
    public function jsonSerialize():string {

        return $this->name;

    }
}
