<?PHP
/**
 * Represents a type of place related to an object group.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);

/**
 * Represents a type of place related to an object group.
 */
enum MDSeriesPlaceRole implements MDValueEnumInterface, JsonSerializable {

    case place_of_production;
    case performance_space;
    case rehearsal_space;

    /**
     * Returns a value of this type based on a string.
     *
     * @param string $input Input to get a value from.
     *
     * @return MDSeriesPlaceRole
     */
    public static function fromString(string $input):MDSeriesPlaceRole {

        return match($input) {
            "place_of_production" => self::place_of_production,
            "performance_space" => self::performance_space,
            "rehearsal_space" => self::rehearsal_space,
            default => throw new MDpageParameterNotFromListException("Unknown series place role"),
        };

    }

    /**
     * Returns a value of this type based on an integer.
     *
     * @param integer $input Input to get a value from.
     *
     * @return MDSeriesPlaceRole
     */
    public static function fromInt(int $input):MDSeriesPlaceRole {

        return match($input) {
            0 => self::place_of_production,
            1 => self::performance_space,
            2 => self::rehearsal_space,
            default => throw new MDpageParameterNotFromListException("Unknown series place role"),
        };

    }

    /**
     * Lists all available names.
     *
     * @return array<string>
     */
    public static function caseNames():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->name;
        }

        return $output;

    }

    /**
     * Gets an unsorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getUnsortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlUnsortedList($tlLoader, self::caseNames(), "series_place_role", "series_place_role");

    }

    /**
     * Gets a sorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getSortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlSortedList($tlLoader, self::caseNames(), "series_place_role", "series_place_role");

    }

    /**
     * Lists all available names.
     *
     * @return array<integer>
     */
    public static function caseInts():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->toInt();
        }

        return $output;

    }

    /**
     * Returns integer representation of object record status.
     *
     * @return integer
     */
    public function toInt():int {

        return match($this) {
            self::place_of_production => 0,
            self::performance_space => 1,
            self::rehearsal_space => 2,
            # default => throw new MDpageParameterNotFromListException("Unknown object record status"),
        };

    }

    /**
     * Returns the name of the current value in translation.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return string
     */
    public function getTledName(MDTlLoader $tlLoader):string {

        return $tlLoader->tl("series_place_role", "series_place_role", $this->name);

    }

    /**
     * Provides the option to serialize as a string during json_encode().
     *
     * @return string
     */
    public function jsonSerialize():string {

        return $this->name;

    }
}
