<?PHP
/**
 * Represents a type of time related to an object group.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);

/**
 * Represents a type of time related to an object group.
 */
enum MDSeriesTimeRole implements MDValueEnumInterface, JsonSerializable {

    case premiere;
    case time_of_presentation;

    /**
     * Returns a value of this type based on a string.
     *
     * @param string $input Input to get a value from.
     *
     * @return MDSeriesTimeRole
     */
    public static function fromString(string $input):MDSeriesTimeRole {

        return match($input) {
            "premiere" => self::premiere,
            "time_of_presentation" => self::time_of_presentation,
            default => throw new MDpageParameterNotFromListException("Unknown series time role"),
        };

    }

    /**
     * Returns a value of this type based on an integer.
     *
     * @param integer $input Input to get a value from.
     *
     * @return MDSeriesTimeRole
     */
    public static function fromInt(int $input):MDSeriesTimeRole {

        return match($input) {
            0 => self::premiere,
            1 => self::time_of_presentation,
            default => throw new MDpageParameterNotFromListException("Unknown series time role"),
        };

    }

    /**
     * Lists all available names.
     *
     * @return array<string>
     */
    public static function caseNames():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->name;
        }

        return $output;

    }

    /**
     * Gets an unsorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getUnsortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlUnsortedList($tlLoader, self::caseNames(), "series_time_role", "series_time_role");

    }

    /**
     * Gets a sorted list of the entries in a translated version.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return array<string, string>
     */
    public static function getSortedList(MDTlLoader $tlLoader):array {
        return MDValueSet::getTlSortedList($tlLoader, self::caseNames(), "series_time_role", "series_time_role");

    }

    /**
     * Lists all available names.
     *
     * @return array<integer>
     */
    public static function caseInts():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->toInt();
        }

        return $output;

    }

    /**
     * Returns integer representation of object record status.
     *
     * @return integer
     */
    public function toInt():int {

        return match($this) {
            self::premiere => 0,
            self::time_of_presentation => 1,
            # default => throw new MDpageParameterNotFromListException("Unknown object record status"),
        };

    }

    /**
     * Returns the name of the current value in translation.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return string
     */
    public function getTledName(MDTlLoader $tlLoader):string {

        return $tlLoader->tl("series_time_role", "series_time_role", $this->name);

    }

    /**
     * Provides the option to serialize as a string during json_encode().
     *
     * @return string
     */
    public function jsonSerialize():string {

        return $this->name;

    }
}
